/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { FloorScene } from "./FloorScene";
import { SPoint } from "@saga-web/draw/lib";
import { SceneMarkItem } from "./items/SceneMarkItem";
import { ZoneItem } from "./items/ZoneItem";
import { Zone } from "./types/Zone";
import { SMouseEvent } from "@saga-web/base/lib";
import { SGraphyItem } from "@saga-web/graphy/lib";

/**
 * 划分业务空间
 *
 * @author 郝建龙
 */
export class DivideFloorScene extends FloorScene {
    /** 是否开启用户标记     */
    _isMarking: boolean = false;
    get isMarking(): boolean {
        return this._isMarking;
    } // Get isMarking
    set isMarking(v: boolean) {
        if (this._isMarking === v) {
            return;
        }
        this._isMarking = v;
    } // Set isMarking

    /** 蒙版item  */
    sceneMark: SGraphyItem | null = null;
    /** 业务空间list   */
    zoneList: ZoneItem[] = [];

    /** 业务空间是否可选  */
    _isZoneSelectable: boolean = true;
    get isZoneSelectable(): boolean {
        return this._isZoneSelectable;
    } // Get isZoneSelectable
    set isZoneSelectable(v: boolean) {
        if (this._isZoneSelectable === v) {
            return;
        }
        this._isZoneSelectable = v;
        this.zoneList.map((t: ZoneItem) => {
            t.selectable = this._isZoneSelectable;
            return t;
        });
    } // Set isZoneSelectable

    /**
     * 构造函数
     *
     * @param   data
     */
    constructor() {
        super();
    } // Constructor

    /**
     * 添加轮廓线
     *
     * @param   data
     */
    addSceneMark(data: SPoint[]): void {
        if (data.length) {
            let sceneMark = new SceneMarkItem(null, data);
            this.addItem(sceneMark);
            this.sceneMark = sceneMark;
        }
    } // Function addSceneMark

    /**
     * 清除蒙版
     *
     */
    clearSceneMark(): void {
        if (this.sceneMark) {
            this.grabItem = null;
            this.removeItem(this.sceneMark);
            this.sceneMark = null;
            this.isMarking = false;
        }
    } // Function clearSceneMark()

    /**
     * 鼠标按下事件
     *
     * @param   event   保存事件参数
     * @return  boolean
     */
    onMouseDown(event: SMouseEvent): boolean {
        if (this.isMarking && event.buttons == 1) {
            if (this.sceneMark) {
                this.sceneMark.onMouseDown(event);
            } else {
                let point = new SPoint(event.x, event.y);
                let sceneMark = new SceneMarkItem(null, point);
                this.addItem(sceneMark);
                this.sceneMark = sceneMark;
                this.grabItem = sceneMark;
            }
        } else {
            super.onMouseDown(event);
        }
        return false;
    } // Function onMouseDown()

    /**
     * 鼠标抬起事件
     *
     * @param   event   保存事件参数
     * @return  boolean
     */
    onMouseUp(event: SMouseEvent): boolean {
        return false;
    } // Function onMouseUp()

    /***
     * 键盘按下事件
     *
     * @param      event   保存事件参数
     */
    onKeyUp(event: KeyboardEvent): void {
        if (this.isMarking) {
            if (this.grabItem) {
                this.grabItem.onKeyUp(event);
            }
        } else {
            super.onKeyUp(event);
        }
    } // Function onKeyUp()

    /***
     * 注册鼠标点击事件
     *
     * @param   _this   接收者
     * @param   fn      处理函数
     */
    click(_this: any, fn: Function): void {
        this.spaceList.map(t => {
            t.connect("click", _this, fn);
        });
        this.zoneList.map(t => {
            t.connect("click", _this, fn);
        });
    } // Function click()

    /**
     *  添加业务空间到scene 中
     *
     *  @param  zone   业务空间list
     */
    addZoneList(zone: Zone[]): void {
        zone.map(t => {
            this.addZone(t);
        });
    } // Function addZoneList

    /**
     *  添加业务空间到scene 中
     *
     *  @param  zone   业务空间
     */
    addZone(zone: Zone): void {
        let item = new ZoneItem(null, zone);
        item.zOrder = 10;
        item.selectable = this.isZoneSelectable;
        this.zoneList.push(item);
        this.addItem(item);
    } // Function addZone

    /**
     *  清空选中的空间
     *
     */
    clearSpaceSelection(): void {
        this.spaceList.map(t => {
            t.selected = false;
            return t;
        });
    } // Function clearSpaceSelection

    /**
     *  清空选中的空间
     *
     */
    clearZoneSelection(): void {
        this.zoneList.map(t => {
            t.selected = false;
            return t;
        });
    } // Function clearZoneSelection

    /**
     *  切换业务空间
     *
     */
    changeSelectZone(zoneitem: ZoneItem): void {
        this.zoneList.map(zone => {
            zone.selected = false;
            zone.unselectFlag = true;
            return zone;
        });
        zoneitem.visible = false;
    } // Function changeSelectZone
} // Class DivideFloorScene
